package com.sun.media.sound;

import javax.sound.sampled.Mixer;
import javax.sound.sampled.spi.MixerProvider;

public final class PortMixerProvider extends MixerProvider
{
    private static PortMixerProvider.PortMixerInfo[] infos;
    private static PortMixer[] devices;
    
    public PortMixerProvider() {
        synchronized (PortMixerProvider.class) {
            if (Platform.isPortsEnabled()) {
                init();
            }
            else {
                PortMixerProvider.infos = new PortMixerProvider.PortMixerInfo[0];
                PortMixerProvider.devices = new PortMixer[0];
            }
        }
    }
    
    private static void init() {
        int nGetNumDevices = nGetNumDevices();
        if (PortMixerProvider.infos == null || PortMixerProvider.infos.length != nGetNumDevices) {
            PortMixerProvider.infos = new PortMixerProvider.PortMixerInfo[nGetNumDevices];
            PortMixerProvider.devices = new PortMixer[nGetNumDevices];
            for (int i = 0; i < PortMixerProvider.infos.length; ++i) {
                PortMixerProvider.infos[i] = nNewPortMixerInfo(i);
            }
        }
    }
    
    @Override
    public Mixer.Info[] getMixerInfo() {
        synchronized (PortMixerProvider.class) {
            Mixer.Info[] array = new Mixer.Info[PortMixerProvider.infos.length];
            System.arraycopy(PortMixerProvider.infos, 0, array, 0, PortMixerProvider.infos.length);
            return array;
        }
    }
    
    @Override
    public Mixer getMixer(Mixer.Info obj) {
        synchronized (PortMixerProvider.class) {
            int i = 0;
            while (i < PortMixerProvider.infos.length) {
                if (PortMixerProvider.infos[i].equals(obj)) {
                    return getDevice(PortMixerProvider.infos[i]);
                }
                ++i;
            }
        }
        throw new IllegalArgumentException("Mixer " + obj.toString() + " not supported by this provider.");
    }
    
    private static Mixer getDevice(PortMixerProvider.PortMixerInfo portMixerInfo) {
        int index = portMixerInfo.getIndex();
        if (PortMixerProvider.devices[index] == null) {
            PortMixerProvider.devices[index] = new PortMixer(portMixerInfo);
        }
        return PortMixerProvider.devices[index];
    }
    
    private static native int nGetNumDevices();
    
    private static native PortMixerProvider.PortMixerInfo nNewPortMixerInfo(int p0);
    
    static {
        Platform.initialize();
    }
    
    static final class PortMixerInfo extends Mixer.Info
    {
        private final int index;
        
        private PortMixerInfo(int index, String str, String vendor, String description, String version) {
            super("Port " + str, vendor, description, version);
            this.index = index;
        }
        
        int getIndex() {
            return this.index;
        }
    }
}
